from .sphinx_base_data_provider import SphinxBaseDataProvider
import re
import codecs


class SphinxHTMLDataProvider(SphinxBaseDataProvider):
    DEFAULT_SOURCE_EXT = '.html'
    DEFAULT_SOURCE_DIR = 'docs/build/html'

    def get_source_data(self, filename):
        with codecs.open(filename, 'r', encoding='utf-8') as f:
            content = f.read()
            title = re.findall(r'<title.*?>(.+?)</title>', content)[0]
            body = re.findall(r'<body.*?>(.+?)</body>', content, re.DOTALL)[0].strip()

        return title, body



import sys
import logging

log = logging.getLogger(__name__)


def setup_logger(level, stream=None):
    levels = [logging.WARNING, logging.INFO, logging.DEBUG]

    if level is None:
        level = 0
    elif level >= len(levels):
        level = len(levels) - 1

    if stream is None:
        stream = sys.stderr

    log.setLevel(levels[level])
    log.addHandler(logging.StreamHandler(stream))


import yaml
from collections import OrderedDict


class OrderedDumper(yaml.Dumper):
    pass


def _dict_representer(dumper, data):
    return dumper.represent_dict(data.iteritems())

OrderedDumper.add_representer(OrderedDict, _dict_representer)


def load(stream):
    return yaml.load(stream)


def dump(data, stream=None):
    return yaml.dump(data, stream=stream, Dumper=OrderedDumper, default_flow_style=False)



import argparse
import copy

from . import log, setup_logger
from .confluence_api import create_confluence_api
from .confluence import ConfluencePageManager, AttachmentPublisher
from .config import ConfigLoader, flatten_page_config_list, PageImageAattachmentConfig
from .constants import DEFAULT_CONFLUENCE_API_VERSION, DEFAULT_WATERMARK_CONTENT
from .data_providers.sphinx_fjson_data_provider import SphinxFJsonDataProvider
from .data_providers.sphinx_html_data_provider import SphinxHTMLDataProvider
from .mutators.page_mutator import WatermarkPageMutator, LinkPageMutator, AnchorPageMutator


def get_data_provider_class(config):
    if config.source_ext == '.html':
        data_provider_class = SphinxHTMLDataProvider
    else:
        data_provider_class = SphinxFJsonDataProvider
    return data_provider_class


def create_publisher(config, confluence_api):
    page_manager = ConfluencePageManager(confluence_api)
    attachment_publisher = AttachmentPublisher(confluence_api)

    data_provider_class = get_data_provider_class(config)

    data_provider = data_provider_class(
        base_dir=config.base_dir,
        downloads_dir=config.downloads_dir,
        images_dir=config.images_dir,
        source_ext=config.source_ext
    )
    return Publisher(config, data_provider, page_manager, attachment_publisher)


class Publisher(object):
    def __init__(self, config, data_provider, page_manager, attachment_manager):
        self._config = config
        self._data_provider = data_provider
        self._page_manager = page_manager
        self._attachment_manager = attachment_manager

    @staticmethod
    def _page_title(current_title, new_title, config_title=None, hold_current=False):
        result = None
        if hold_current:
            result = current_title
        elif config_title:
            result = config_title
        elif new_title:
            result = new_title
        else:
            result = current_title
        return result

    def _page(self, current_page, source):
        page = copy.copy(current_page)
        page.title, page.body = self._data_provider.get_source_data(self._data_provider.get_source(source))
        return page

    def _page_attachment_file(self, attachment_config):
        if isinstance(attachment_config, PageImageAattachmentConfig):
            return self._data_provider.get_image(attachment_config.path)
        return self._data_provider.get_attachment(attachment_config.path)

    @staticmethod
    def _remove_page_mutators(page, mutators):
        for mutator in mutators:
            mutator.apply_backward(page)

    @staticmethod
    def _add_page_mutators(page, mutators):
        for mutator in mutators:
            mutator.apply_forward(page)

    def _init_page_mutators(self, page_config, old_title, hold_titles):
        mutators = []

        if page_config.link:
            mutators.append(LinkPageMutator(page_config.link))
        if page_config.watermark:
            mutators.append(WatermarkPageMutator(page_config.watermark))
        if hold_titles:
            mutators.append(AnchorPageMutator(old_title))

        return mutators

    def _pages_to_update(self, force=False, watermark=False, hold_titles=False):
        pages_to_update = []
        for page_config in flatten_page_config_list(self._config.pages):
            if page_config.id is None:
                raise AttributeError('Missed attribute "id"')
            current_page = self._page_manager.load(page_config.id)
            page = self._page(current_page, page_config.source)

            mutators = self._init_page_mutators(page_config, page.title, hold_titles)
            self._remove_page_mutators(current_page, mutators)

            page.title = self._page_title(current_page.title, page.title, page_config.title, hold_titles)
            if not force and current_page == page:
                continue

            self._add_page_mutators(page, mutators)

            pages_to_update.append(page)
        return pages_to_update

    def _attachments_to_update(self, force=False):
        attachments_to_update = []
        for page_config in flatten_page_config_list(self._config.pages):
            for attachment_config in page_config.images + page_config.downloads:
                page_attachment = (page_config.id, self._page_attachment_file(attachment_config))
                attachments_to_update.append(page_attachment)
        return attachments_to_update

    def publish(self, force=False, watermark=False, hold_titles=False):
        pages_to_update = self._pages_to_update(force, watermark, hold_titles)
        attachments_to_update = self._attachments_to_update(force)

        log.info('Publishing pages...')
        self._publish_pages(pages_to_update)

        log.info('Publishing attachments...')
        self._publish_attachments(attachments_to_update)

    def _publish_pages(self, pages):
        for page in pages:
            self._publish_page(page)

    def _publish_page(self, page):
        log.info('Publishing page: id: %s' % page.id)
        content_id = self._page_manager.update(page)
        log.info('Published to: %s' % content_id)

    def _publish_attachments(self, attachments):
        for content_id, attachment in attachments:
            self._publish_page_attachement(content_id, attachment)

    def _publish_page_attachement(self, content_id, filename):
        log.info('Publishing attachment: %s (parent_id: %s)' % (filename, content_id))
        self._attachment_manager.publish(content_id, filename)
        log.info('Published to: %s' % content_id)


def setup_config_overrides(config, url=None, watermark=None, link=None):
    if url:
        config.url = url

    if watermark:
        for page in flatten_page_config_list(config.pages):
            if watermark.lower() == 'false':
                page.watermark = None
            elif watermark.lower() == 'true':
                page.watermark = DEFAULT_WATERMARK_CONTENT
            else:
                page.watermark = watermark

    if link:
        for page in flatten_page_config_list(config.pages):
            if link.lower() == 'false':
                page.link = None
            else:
                page.link = link


def main():
    parser = argparse.ArgumentParser(description='Publish documentation (Sphinx fjson) to Confluence')
    parser.add_argument('config', type=str, help='Configuration file')
    parser.add_argument('-u', '--url', type=str, help='Confluence Url')
    parser.add_argument('-a', '--auth', type=str, help='Base64 encoded user:password string')
    parser.add_argument('-F', '--force', action='store_true', help='Publish not changed page.')
    parser.add_argument('-w', '--watermark', type=str, help='Overrides the watermarks. Also can be "False" to remove '
                                                            'all watermarks; or "True" to add watermarks'
                                                            'with default text: "{}" on all pages.'.format(DEFAULT_WATERMARK_CONTENT))
    parser.add_argument('-l', '--link', type=str, help='Overrides page link. If value is "False" then removes the link.')
    parser.add_argument('-ht', '--hold-titles', action='store_true', help='Do not change page titles while publishing.')
    parser.add_argument('-v', '--verbose', action='count')

    args = parser.parse_args()

    setup_logger(args.verbose)

    config = ConfigLoader.from_yaml(args.config)

    setup_config_overrides(config, args.url, args.watermark, args.link)

    confluence_api = create_confluence_api(DEFAULT_CONFLUENCE_API_VERSION, config.url, args.auth)
    publisher = create_publisher(config, confluence_api)
    publisher.publish(args.force, args.watermark, args.hold_titles)

if __name__ == '__main__':
    main()



import os
import copy
from operator import attrgetter

try:
    from lxml import etree
except ImportError:
    import xml.etree.ElementTree as etree


class Content(object):
    type = None

    def __init__(self):
        self.id = None

    def __eq__(self, other):
        return self.__dict__ == other.__dict__


class Attachement(Content):
    type = 'attachment'

    def __init__(self):
        self.title = ''
        self.media_type = ''
        super(Attachement, self).__init__()


class ImageAttachement(Attachement):
    pass


class DownloadAttachement(Attachement):
    pass


class Page(Content):
    type = 'page'

    def __init__(self):
        self.version_number = 0
        self.space_key = None
        self.ancestors = list()
        self.body = None
        self.title = None
        self.unused_title = None
        super(Page, self).__init__()

    def __eq__(self, other):
        first = copy.copy(self.__dict__)
        second = copy.copy(other.__dict__)

        del first['ancestors']
        del second['ancestors']
        if len(self.ancestors) != len(other.ancestors):
            return False

        for first_ancestor, second_ancestor in zip(self.ancestors, other.ancestors):
            if not (first_ancestor == second_ancestor):
                return False

        del first['body']
        del second['body']
        if not PageBodyComparator.is_equal(self.body, other.body):
            return False

        return first == second


class Ancestor(Content):
    type = 'page'


class ConfluenceManager(object):
    def __init__(self, api):
        self._api = api


class ConfluencePageManager(ConfluenceManager):

    def load(self, content_id):
        data = self._api.get_content(content_id, 'ancestors,version,space,body.storage')
        p = Page()
        p.id = data['id']
        p.type = data['type']
        p.version_number = data['version']['number']
        p.space_key = data['space']['key']
        p.title = data['title']
        p.body = data['body']['storage']['value']

        for ancestor_data in data['ancestors']:
            ancestor = Ancestor()
            ancestor.id = ancestor_data['id']
            ancestor.type = ancestor_data['type']
            p.ancestors.append(ancestor)

        return p

    def create(self, page):
        ancestor = page.ancestors[-1]

        data = self._page_payload(page.space_key, page.body, page.title,
                                  ancestor_id=ancestor.id, ancestor_type=ancestor.type,)
        ret = self._api.create_content(data)
        page.id = ret['id']
        return page.id

    def update(self, page, bump_version=True):
        if bump_version:
            page.version_number += 1

        ancestor = page.ancestors[-1]

        data = self._page_payload(page.space_key, page.body, page.title,
                                  ancestor_id=ancestor.id, ancestor_type=ancestor.type,
                                  content_id=page.id, version=page.version_number)
        ret = self._api.update_content(page.id, data)
        page.id = ret['id']
        return page.id

    @staticmethod
    def _page_payload(space_key, body=None, title=None,
                      ancestor_id=None, ancestor_type='page',
                      content_id=None, version=None, content_type='page'):
        payload = {
            'type': content_type,
            'space': {
                'key': space_key
            },
        }

        if body:
            payload['body'] = {
                'storage': {
                    'value': body,
                    'representation': 'storage'
                }
            }

        if ancestor_id:
            payload['ancestors'] = [
                {
                    'type': ancestor_type,
                    'id': ancestor_id,
                }
            ]

        if content_id:
            payload['id'] = content_id

        if title:
            payload['title'] = title

        if version:
            payload['version'] = {
                'number': version
            }

        return payload


class AttachmentPublisher(ConfluenceManager):
    def publish(self, content_id, filepath):
        attachments = self._get_page_metadata(content_id)
        filename = os.path.basename(filepath)

        if filename in map(attrgetter('title'), attachments):
            # TODO: fixme. skipping if file already exists. its ugly hack
            return

        with open(filepath, 'rb') as f:
            self._api.create_attachment(content_id, f)

    @staticmethod
    def _parse_attachments(data):
        attachments = []
        for attachment_data in data['children']['attachment']['results']:
            media_type = attachment_data['metadata']['mediaType']
            attachment_class = ImageAttachement if 'image' in media_type else DownloadAttachement
            attachment = attachment_class()
            attachment.id = attachment_data['id']
            attachment.title = attachment_data['title']
            attachment.media_type = media_type

            attachments.append(attachment)

        return attachments

    def _get_page_metadata(self, content_id):
        data = self._api.get_content(content_id, 'children.attachment')

        page_attachments = self._parse_attachments(data)

        return page_attachments


class PageBodyComparator(object):

    @classmethod
    def is_equal(cls, first, second):
        if first == '' and second == '':
            return True

        if first == '' and second != '' or first != '' and second == '':
            return False

        # 'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        # 'xsi:schemaLocation="http://www.atlassian.com/schema/confluence/4/ac/ confluence.xsd" '
        wrapper = u'<?xml version="1.0" encoding="UTF-8"?>' \
                  u'<!DOCTYPE ac:confluence SYSTEM "confluence.dtd">' \
                  u'<ac:confluence xmlns:ac="http://www.atlassian.com/schema/confluence/4/ac/" ' \
                  u'xmlns:ri="http://www.atlassian.com/schema/confluence/4/ri/">{}</ac:confluence>'

        first_xml = etree.XML(wrapper.format(first).encode(encoding='utf-8'), parser=cls._parser())
        second_xml = etree.XML(wrapper.format(second).encode(encoding='utf-8'), parser=cls._parser())
        return cls._elements_equal(first_xml, second_xml)

    @staticmethod
    def _parser():
        # use lxml HTMLParser if it exists
        if hasattr(etree, 'HTMLParser'):
            return etree.HTMLParser()

        # or xml.etree.ElementTree.XMLParser
        # fix unknown entity
        # http://stackoverflow.com/questions/7237466/python-elementtree-support-for-parsing-unknown-xml-entities
        parser = etree.XMLParser()
        parser.entity['nbsp'] = 'nbsp'
        return parser

    @classmethod
    def _elements_equal(cls, e1, e2):
        if e1.tag != e2.tag:
            return False
        if e1.text != e2.text:
            return False
        if not cls._attributes_equals(e1, e2):
            return False
        if len(e1) != len(e2):
            return False
        return all(cls._elements_equal(c1, c2) for c1, c2 in zip(e1, e2))

    @staticmethod
    def _attributes_equals(e1, e2):
        # confluence create additional attributes for structured macros
        if 'structured-macro' == e1.tag:
            return e1.attrib.get('name') == e2.attrib.get('name')
        elif 'structured-macro' in e1.tag:
            confluence_ac_attribute_name = '{http://www.atlassian.com/schema/confluence/4/ac/}name'
            return e1.attrib.get(confluence_ac_attribute_name) == e2.attrib.get(confluence_ac_attribute_name)

        return e1.attrib == e2.attrib



import os
from .sphinx_base_data_provider import SphinxBaseDataProvider
from ..serializers import json_serializer


class SphinxFJsonDataProvider(SphinxBaseDataProvider):
    DEFAULT_SOURCE_EXT = '.fjson'
    DEFAULT_SOURCE_DIR = 'docs/build/json'

    def get_source_data(self, filename):
        if os.path.isabs(filename):
            filepath = filename
        else:
            filepath = self.get_source(filename)
        with open(filepath, 'r') as f:
            content = json_serializer.load(f)

        return content.get('title'), content.get('body')



import argparse

from . import log, setup_logger
from .confluence_api import create_confluence_api
from .constants import DEFAULT_CONFLUENCE_API_VERSION
from .config import ConfigLoader, ConfigDumper
from .confluence import ConfluencePageManager, Page, Ancestor


def setup_config_overrides(config, url=None):
    if url:
        config.url = url


def empty_page(space_key, title, ancestor_id, ancestor_type):
    ancestor = Ancestor()
    ancestor.id = ancestor_id
    ancestor.type = ancestor_type

    page = Page()
    page.space_key = space_key
    page.title = title
    page.body = ''
    page.ancestors.append(ancestor)

    return page


def make_page(parent_page, title, page_manager):
    page = empty_page(parent_page.space_key, title, parent_page.id, parent_page.type)
    page_id = page_manager.create(page)
    return int(page_id)


def make_pages(config, page_manager, parent_id=None):
    parent_page = None
    if parent_id:
        parent_page = page_manager.load(parent_id)

    for page_config in config.pages:
        if not page_config.id:
            if not parent_page:
                log.warning('Page without id and parent page. Skip.')
            elif not page_config.title:
                log.warning('Page without title. Skip. Parent page id: {parent_id}.'.format(parent_id=parent_id))
            else:
                page_config.id = make_page(parent_page, page_config.title, page_manager)
                log.info('Page with id {page_id} has been created. Parent page id: {parent_id}'
                         .format(page_id=page_config.id, parent_id=parent_page.id))
        else:
            log.info('Skip page with id {page_id}'.format(page_id=page_config.id))

        if len(page_config.pages):
            make_pages(page_config, page_manager, page_config.id)


def main():
    parser = argparse.ArgumentParser(description='Create Confluence pages and update configuration file with it ids')
    parser.add_argument('config', type=str, help='Configuration file')
    parser.add_argument('-u', '--url', type=str, help='Confluence Url')
    parser.add_argument('-a', '--auth', type=str, required=True, help='Base64 encoded user:password string. Required.')
    parser.add_argument('-pid', '--parent-id', type=str, help='Parent page ID in confluence.')
    parser.add_argument('-v', '--verbose', action='count')

    args = parser.parse_args()
    setup_logger(args.verbose)

    config = ConfigLoader.from_yaml(args.config)
    setup_config_overrides(config, args.url)

    confluence_api = create_confluence_api(DEFAULT_CONFLUENCE_API_VERSION, config.url, args.auth)
    page_manager = ConfluencePageManager(confluence_api)
    make_pages(config, page_manager, args.parent_id)

    ConfigDumper.to_yaml_file(config, args.config)
    log.info('Config has been updated.')

if __name__ == '__main__':
    main()



import os

from . import DataProvider


class SphinxBaseDataProvider(DataProvider):

    DEFAULT_SOURCE_EXT = None
    DEFAULT_ROOT_DIR = './'
    DEFAULT_SOURCE_DIR = 'docs/build/result'
    DEFAULT_IMAGES_DIR = '_images'
    DEFAULT_DOWNLOADS_DIR = '_downloads'

    def __init__(self, root_dir=None, base_dir=None, downloads_dir=None, images_dir=None, source_ext=None):
        self._root_dir = os.path.abspath(root_dir or './')

        self._source_dir = base_dir or self.DEFAULT_SOURCE_DIR
        if not os.path.isabs(self._source_dir):
            self._source_dir = os.path.join(self._root_dir, self._source_dir)

        self._downloads_dir = downloads_dir or self.DEFAULT_DOWNLOADS_DIR
        if not os.path.isabs(self._downloads_dir):
            self._downloads_dir = os.path.join(self._source_dir, self._downloads_dir)

        self._images_dir = images_dir or self.DEFAULT_IMAGES_DIR
        if not os.path.isabs(self._images_dir):
            self._images_dir = os.path.join(self._source_dir, self._images_dir)

        self._source_ext = source_ext or self.DEFAULT_SOURCE_EXT

    def get_source(self, filename):
        return os.path.join(self._source_dir, filename + self._source_ext)

    def get_source_data(self, filename):
        raise NotImplementedError()

    def get_image(self, filename):
        return os.path.join(self._images_dir, filename)

    def get_attachment(self, filename):
        return os.path.join(self._downloads_dir, filename)



print("import: 'conf_publisher'")
import conf_publisher

print("import: 'conf_publisher.page_dumper'")
import conf_publisher.page_dumper

print("import: 'conf_publisher.page_maker'")
import conf_publisher.page_maker

print("import: 'conf_publisher.publish'")
import conf_publisher.publish




import re


class PageMutator(object):

    def apply_forward(self, page):
        raise NotImplemented()

    def apply_backward(self, page):
        raise NotImplemented()


class TemplatePageMutator(PageMutator):
    template_prefix = None
    template_suffix = None
    template = None

    def __init__(self):
        self.template_params = {}

    def set_param(self, name, value):
        self.template_params[name] = value

    def apply_forward(self, page):
        page.body = self.template_prefix + self.template.format(**self.template_params) + self.template_suffix + page.body

    def apply_backward(self, page):
        page.body = re.sub(re.escape(self.template_prefix) + '.*' + re.escape(self.template_suffix), '', page.body, flags=re.DOTALL)


class WatermarkPageMutator(TemplatePageMutator):
    template_prefix = '<span class="WATERMARK BEGIN"> </span>'
    template_suffix = '<span class="WATERMARK END"> </span>'
    template = """<ac:structured-macro ac:name="info">
        <ac:rich-text-body>
        <p><span>{watermark}</span></p>
        </ac:rich-text-body>
        </ac:structured-macro>"""

    def __init__(self, watermark):
        super(WatermarkPageMutator, self).__init__()
        self.set_param('watermark', watermark)


class LinkPageMutator(TemplatePageMutator):
    template_prefix = '<span class="LINK BEGIN"> </span>'
    template_suffix = '<span class="LINK END"> </span>'
    template = """<ac:structured-macro ac:name="info">
        <ac:rich-text-body>
        <p><span><a href="{link}" _blank="true">{link}</a></span></p>
        </ac:rich-text-body>
        </ac:structured-macro>"""

    def __init__(self, link):
        super(LinkPageMutator, self).__init__()
        self.set_param('link', link)


class AnchorPageMutator(PageMutator):
    anchor_expression = None
    _old_title = u''

    def __init__(self, old_title):
        if old_title:
            self._old_title = ''.join(old_title.split())
            self.anchor_expression = re.compile(re.escape(self._old_title))

    def apply_forward(self, page):
        if not page.title or not page.body or not self.anchor_expression:
            return
        _title = ''.join(page.title.split())
        page.body = self.anchor_expression.sub(_title, page.body, re.I|re.U)

    def apply_backward(self, page):
        pass



import copy
from collections import OrderedDict

from .errors import ConfigError
from .serializers import yaml_serializer


class Config(object):
    def __init__(self):
        self.url = None
        self.base_dir = None
        self.downloads_dir = None
        self.images_dir = None
        self.source_ext = None
        self.pages = list()

    def __eq__(self, other):
        first = copy.copy(self.__dict__)
        del first['pages']

        second = copy.copy(other.__dict__)
        del second['pages']

        if len(self.pages) != len(other.pages):
            return False

        for first_page, second_page in zip(self.pages, other.pages):
            if not (first_page == second_page):
                return False

        return first == second


class PageConfig(object):
    def __init__(self):
        self.id = None

        self.title = None
        self.source = None
        self.link = None
        self.watermark = None

        self.images = list()
        self.downloads = list()

        self.pages = list()

    def __eq__(self, other):
        first = copy.copy(self.__dict__)
        del first['images']
        del first['downloads']

        second = copy.copy(other.__dict__)
        del second['images']
        del second['downloads']

        if len(self.images) != len(other.images):
            return False

        if len(self.downloads) != len(other.downloads):
            return False

        for first_attach, second_attach in zip(self.images, other.images):
            if not (first_attach == second_attach):
                return False

        for first_attach, second_attach in zip(self.downloads, other.downloads):
            if not (first_attach == second_attach):
                return False

        return first == second


class PageAattachmentConfig(object):
    TYPE_ATTACHMENT = 'attachment'
    TYPE_IMAGE = 'image'

    def __init__(self):
        self.type = self.TYPE_ATTACHMENT
        self.path = None

    def __eq__(self, other):
        return self.__dict__ == other.__dict__


class PageImageAattachmentConfig(PageAattachmentConfig):
    def __init__(self):
        self.type = self.TYPE_IMAGE
        super(PageImageAattachmentConfig, self).__init__()


class ConfigLoader:

    @classmethod
    def from_yaml(cls, config_path):
        with open(config_path, 'rb') as f:
            config = yaml_serializer.load(f)
        return cls.from_dict(config)

    @classmethod
    def from_dict(cls, config_dict):
        if 'version' not in config_dict:
            raise ConfigError('`version` param is required')

        if config_dict['version'] != 2:
            raise ConfigError('invalid config version. required: 2')

        config = Config()

        for attr in ('url', 'base_dir', 'downloads_dir', 'images_dir', 'source_ext'):
            if attr in config_dict:
                setattr(config, attr, config_dict[attr])

        config.pages = cls._pages_from_list(config_dict.get('pages', list()))

        return config

    @classmethod
    def _pages_from_list(cls, pages_list):
        pages = list()
        for page_dict in pages_list:
            pages.append(cls._page_from_dict(page_dict))
        return pages

    @classmethod
    def _page_from_dict(cls, page_dict):
        page_config = PageConfig()

        for attr in ('id', 'title', 'source', 'link', 'watermark'):
            if attr in page_dict:
                setattr(page_config, attr, page_dict[attr])

        if 'attachments' in page_dict:
            for path in page_dict['attachments'].get('images', list()):
                page_config.images.append(cls._attach_from_path(path, PageImageAattachmentConfig))
            for path in page_dict['attachments'].get('downloads', list()):
                page_config.downloads.append(cls._attach_from_path(path, PageAattachmentConfig))

        page_config.pages = cls._pages_from_list(page_dict.get('pages', list()))

        return page_config

    @staticmethod
    def _attach_from_path(attach_path, attach_class):
        attach = attach_class()
        attach.path = attach_path
        return attach


class ConfigDumper:
    @classmethod
    def to_yaml_file(cls, config, config_path):
        with open(config_path, 'w') as f:
            cls.to_yaml_string(config, stream=f)

    @classmethod
    def to_yaml_string(cls, config, stream=None):
        return yaml_serializer.dump(cls.to_dict(config), stream=stream)

    @classmethod
    def to_dict(cls, config):
        config_dict = OrderedDict(version=2)

        for attr in ('url', 'base_dir', 'downloads_dir', 'images_dir', 'source_ext'):
            attr_value = getattr(config, attr)
            if attr_value:
                config_dict[attr] = attr_value

        config_dict['pages'] = cls._pages_to_list(config.pages)

        return config_dict

    @classmethod
    def _pages_to_list(cls, pages_config):
        pages = list()
        for page_config in pages_config:
            pages.append(cls._page_to_dict(page_config))
        return pages

    @classmethod
    def _page_to_dict(cls, page_config):
        page_dict = OrderedDict()

        for attr in ('id', 'title', 'source', 'link', 'watermark'):
            attr_value = getattr(page_config, attr)
            if attr_value:
                page_dict[attr] = attr_value

        if len(page_config.images) + len(page_config.downloads):
            page_dict['attachments'] = OrderedDict()
        if len(page_config.images):
            page_dict['attachments']['images'] = cls._attaches_to_path(page_config.images)
        if len(page_config.images):
            page_dict['attachments']['downloads'] = cls._attaches_to_path(page_config.downloads)

        pages = cls._pages_to_list(page_config.pages)
        if len(pages):
            page_dict['pages'] = pages

        return page_dict

    @classmethod
    def _attaches_to_path(cls, attaches):
        attaches_list = []
        for attach_config in attaches:
            attaches_list.append(cls._attach_to_path(attach_config))
        return attaches_list

    @staticmethod
    def _attach_to_path(attach_config):
        return attach_config.path


def flatten_page_config_list(pages):
    for page in pages:
        yield page
        for subpage in flatten_page_config_list(page.pages):
            yield subpage



class DataProvider(object):

    def get_source(self, filename):
        raise NotImplemented()

    def get_source_data(self, filename):
        raise NotImplemented()

    def get_image(self, filename):
        raise NotImplemented()

    def get_image_stream(self, filename):
        raise NotImplemented()

    def get_attachment(self, filename):
        raise NotImplemented()

    def get_attachment_stream(self, filename):
        raise NotImplemented()



import argparse
import codecs
import sys

from .confluence_api import create_confluence_api
from .confluence import ConfluencePageManager
from .constants import DEFAULT_CONFLUENCE_API_VERSION


def main():
    parser = argparse.ArgumentParser(description='Dumps Confluence page in storage format')
    parser.add_argument('page_id', type=str, help='Configuration file')
    parser.add_argument('-u', '--url', type=str, help='Confluence Url')
    parser.add_argument('-a', '--auth', type=str, help='Base64 encoded user:password string')
    parser.add_argument('-o', '--output', type=str, help='Output file|stdout|stderr', default='stdout')

    args = parser.parse_args()

    confluence_api = create_confluence_api(DEFAULT_CONFLUENCE_API_VERSION, args.url, args.auth)
    page_manager = ConfluencePageManager(confluence_api)
    page = page_manager.load(args.page_id)

    if args.output.lower() == 'stdout':
        f = sys.stdout
    elif args.output.lower() == 'stderr':
        f = sys.stderr
    else:
        f = codecs.open(args.output, 'w', encoding='utf-8')

    with f:
        f.write(page.body)


if __name__ == '__main__':
    main()



import requests
from . import log
from .constants import DEFAULT_CONFLUENCE_API_VERSION


def create_confluence_api(version, url, auth):
    # Documentation for different REST API versions: https://docs.atlassian.com/confluence/REST/

    confluence_api_class = None
    if version == DEFAULT_CONFLUENCE_API_VERSION:
        confluence_api_class = ConfluenceRestApi553

    if confluence_api_class is None:
        raise NotImplementedError('This API Version is not implemented')

    confluence_api = confluence_api_class(url, auth)

    return confluence_api


class ConfluenceRestApiBase(object):
    api_path = 'rest/api'

    def __init__(self, url, auth):
        self.confluence_url = url.rstrip('/')
        self.auth = auth
        self.headers = {
            'content-type': 'application/json',
            'Authorization': 'Basic ' + auth
        }

    @staticmethod
    def _build_params(params_map):
        return dict((k, v) for k, v in params_map.items() if v is not None)

    def _construct_url(self, *url_parts):
        parts = [str(part) for part in url_parts]
        return '/'.join([self.confluence_url, self.api_path] + parts)

    def _get(self, url, **kwargs):
        return self._request(requests.get, url, **kwargs)

    def _post(self, url, _json=None, **kwargs):
        return self._request(requests.post, url, json=_json, **kwargs)

    def _put(self, url, _json=None, **kwargs):
        return self._request(requests.put, url, json=_json, **kwargs)

    def _delete(self, url, **kwargs):
        return self._request(requests.delete, url, **kwargs)

    def _request(self, requester, url, **kwargs):
        if 'headers' not in kwargs:
            kwargs['headers'] = self.headers

        log.debug('Request URL: %s', url)
        log.debug('Request Arguments: %s', kwargs)

        r = requester(url, **kwargs)
        if r.status_code != requests.codes.ok:
            log.error(r.content)
            r.raise_for_status()

        ret = r.json()

        log.debug('Request Response: %s', ret)

        return ret


class ConfluenceRestApi553(ConfluenceRestApiBase):
    # Documentation for Confluence v5.5.3 REST API: https://docs.atlassian.com/confluence/REST/5.5.3/

    def list_content(self, space_key, type='page', title=None, posting_day=None, expand='history,space,version', start=0, limit=25):
        """
        Returns a paginated list of Content

        GET /rest/api/content?type&spaceKey&title&postingDay&expand&start&limit

        :param space_key:       the space key to find content under. Required
        :param type:            the content type to return. Default value: page. Valid values: page, blogpost. Default: page
        :param title:           the title of the page to find. Required for page type
        :param posting_day:     the posting day of the blog post. Required for blogpost type. Format: yyyy-mm-dd. Example: 2013-02-13
        :param expand:          a comma separated list of properties to expand on the content. Default value: history,space,version.
        :param start:           the start point of the collection to return
        :param limit:           the limit of the number of items to return, this may be restricted by fixed system limits. Default: 25
        :return:
        """
        params_map = {'spaceKey': space_key, 'type': type, 'title': title, 'postingDay': posting_day, 'expand': expand, 'start': start, 'limit': limit}

        url = self._construct_url('content')
        params = self._build_params(params_map)

        ret = self._get(url, params=params)
        return ret

    def get_content(self, content_id, expand='history,space,version'):
        """
        Returns a piece of Content.

        GET /rest/api/content/{id}

        :param content_id:      the id of the content
        :param expand:          A comma separated list of properties to expand on the content. Default value: history,space,version
        :return:
        """
        params_map = {'expand': expand}

        url = self._construct_url('content', content_id)
        params = self._build_params(params_map)

        ret = self._get(url, params=params)
        return ret

    def create_content(self, data):
        """
        Creates a new piece of Content.

        POST /rest/api/content

        :param data:
        :return:
        """
        url = self._construct_url('content')
        ret = self._post(url, data)
        return ret

    def update_content(self, content_id, data):
        """
        Updates a piece of Content.

        PUT /rest/api/content/{id}

        :param content_id:      the id of the content
        :return:

        The body contains the representation of the content. Must include the new version number.

        """
        url = self._construct_url('content', content_id)
        ret = self._put(url, data)
        return ret

    def delete_content(self, content_id):
        """
        Deletes a piece of Content.

        DELETE /rest/api/content/{id}

        :param content_id:      the id of the content
        :return:
        """
        url = self._construct_url('content', content_id)
        ret = self._delete(url)
        return ret

    def list_attachments(self, content_id, expand=None, start=0, limit=50, filename=None, media_type=None):
        """
        Returns a paginated list of attachment Content entities within a single container.

        GET /rest/api/content/{id}/child/attachment?expand&start&limit&filename&mediaType

        :param content_id:      a string containing the id of the attachments content container
        :param expand:          a comma separated list of properties to expand on the Attachments returned. Optional.
        :param start:           the index of the first item within the result set that should be returned. Optional.
        :param limit:           how many items should be returned after the start index. Optional. Default: 50
        :param filename:        filter parameter to return only the Attachment with the matching file name. Optional.
        :param media_type:      filter parameter to return only Attachments with a matching Media-Type. Optional.
        :return:
        """
        params_map = {'mediaType': media_type, 'filename': filename, 'expand': expand, 'start': start, 'limit': limit}

        url = self._construct_url('content', content_id, 'child', 'attachment')
        params = self._build_params(params_map)

        ret = self._get(url, params=params)
        return ret

    def create_attachment(self, content_id, attachment, comment=None, minor_edits=False):
        """
        Add one or more attachments to a Confluence Content entity, with optional comments.

        POST /rest/api/content/{id}/child/attachment

        :param content_id:      a string containing the id of the attachments content container
        :param attachment:
        :param comment:
        :param minor_edits:
        :return:
        """
        url = self._construct_url('content', content_id, 'child', 'attachment')
        ret = self._create_attachment(url, attachment, comment, minor_edits)
        return ret

    def update_attachment_data(self, content_id, attachment_id, attachment, comment=None, minor_edits=False):
        """
        Update the binary data of an Attachment, and optionally the comment.

        POST /rest/api/content/{id}/child/attachment/{attachmentId}/data

        :param content_id:      a string containing the id of the attachments content container
        :param attachment_id:   the id of the attachment to update
        :param attachment:
        :param comment:
        :param minor_edits:
        :return:
        """
        url = self._construct_url('content', content_id, 'child', 'attachment', attachment_id, 'data')
        ret = self._create_attachment(url, attachment, comment, minor_edits)
        return ret

    def _create_attachment(self, url, attachment, comment=None, minor_edits=False):
        headers = {
            'X-Atlassian-Token': 'no-check',
            'Authorization': 'Basic ' + self.auth
        }

        params_map = {'comment': comment, 'minorEdit': minor_edits}
        params = self._build_params(params_map)

        ret = self._post(url, files={'file': attachment}, data=params, headers=headers)
        return ret
